import _winreg, struct
from wxPython.wx import *

### Code to read info from Creative sound drivers.

# Registry key name extracted from Creative driver 5.12.1.383 date 2003-04-11
cbRoot='SYSTEM\\CurrentControlSet\\Control\\Creative Tech\\Emu10kx\\DevCon'
cbGuid='{00000000-00000000-0000000F-00001102-00000004-10021102}'
cbInfo='CurrentSpeakerInfo'
cbKey = cbRoot + '\\' + cbGuid + '\\' + cbInfo

def cStrToPythonStr(s):
    return s[:s.index('\x00')]

def readName():
    "Read the speaker type that the Creative Sound driver is configured for"

    # Read out the registry value and convert it to a Python string
    regkey = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, cbKey)
    name = _winreg.QueryValueEx(regkey, 'Name')[0]
    _winreg.CloseKey(regkey)
    return cStrToPythonStr(name)

def readAllInfo():
    """Read all configuration data from the Creative Sound driver.
    Returns a list of tuples. Each tuple is a pair: name, value"""
    regkey = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, cbKey)
    numValues = _winreg.QueryInfoKey(regkey)[1]
    r = []
    for n in range(0, numValues):
        (name, value, type) = _winreg.EnumValue(regkey, n)
        if (name == 'Name'):
            r.append((name, cStrToPythonStr(value)))
        elif (name == 'SpkGuid '):
            r.append((name, value))
        else:
            r.append((name, struct.unpack('i', value)[0]))
    _winreg.CloseKey(regkey)
    return r

### GUI code. References:
# http://radio.weblogs.com/0110159/gems/systray_py.txt
# http://wiki.wxpython.org/index.cgi/FlashingTaskbarIcon

iconMap = {}
iconMap['6.1 Speakers'] = wxIcon('6.ico', wxBITMAP_TYPE_ICO)
iconMap['5.1 Speakers'] = wxIcon('5.ico', wxBITMAP_TYPE_ICO)
iconMap['4/4.1 Speakers'] = wxIcon('4.ico', wxBITMAP_TYPE_ICO)
iconMap['2/2.1 Speakers'] = wxIcon('2.ico', wxBITMAP_TYPE_ICO)
iconMap['Headphones'] = wxIcon('h.ico', wxBITMAP_TYPE_ICO)

aboutString = """Speaker Watch 2003-08-23
by Nelson Minar <nelson@monkey.org>
http://www.nelson.monkey.org/~nelson/weblog/

Watches the Creative sound driver status
Updates icon based on speaker configuration
"""

class TrayThing(wxFrame):
    TBMENU_ABOUT = 1000
    TBMENU_CLOSE = 1001
    ID_ICON_TIMER = 1002
    lastData = None
    updateInterval = 15000    # milliseconds
    
    def __init__(self, parent, id, title):
        # Create a window (we'll never use it)
        wxFrame.__init__(self, parent, -1, title, size = (1, 1),
                         style=wxFRAME_NO_TASKBAR|wxNO_FULL_REPAINT_ON_RESIZE)

        # Create a system tray icon, fill it in
        self.tbicon = wxTaskBarIcon()
        self.Update()

        # Register GUI event handlers
        EVT_TASKBAR_RIGHT_UP(self.tbicon, self.OnTaskBarMenu)
        EVT_MENU(self.tbicon, self.TBMENU_CLOSE, self.OnTaskBarClose)
        EVT_MENU(self.tbicon, self.TBMENU_ABOUT, self.OnTaskBarAbout)

        # Create and register a timer to check settings periodically
        self.icontimer = wxTimer(self, self.ID_ICON_TIMER)
        EVT_TIMER(self, self.ID_ICON_TIMER, self.Update)
        self.icontimer.Start(self.updateInterval)

    def Update(self, ev=None):
        "Update the icon based on current data"
        data = readAllInfo()
        if (data != self.lastData):
            name = data[0][1]
            tooltip = "%2d %s\n%2d %s\n%2d %s" % \
                      (data[1][1], data[1][0][4:],
                       data[2][1], data[2][0][4:],
                       data[3][1], data[3][0][4:])
            self.tbicon.SetIcon(iconMap[name], tooltip)
        self.lastData = data

    def OnCloseWindow(self, event):
        if hasattr(self, "tbicon"):
            del self.tbicon
        self.Destroy()

    def OnTaskBarMenu(self, evt):
        menu = wxMenu()
        menu.Append(self.TBMENU_ABOUT, "About")
        menu.Append(self.TBMENU_CLOSE, "Close")
        self.tbicon.PopupMenu(menu)
        menu.Destroy()

    def OnTaskBarAbout(self, evt):
        box=wxMessageDialog(self, aboutString, 'About Speaker Watch', wxOK)
        box.ShowModal()
        box.Destroy()
        
    def OnTaskBarClose(self, evt):
        self.Close()
        wxGetApp().ProcessIdle()

class MyApp(wxApp):
    def OnInit(self):
        frame = TrayThing(None, -1, "Speaker Watch")
        return true

def main():
    app = MyApp()
    app.MainLoop()

if __name__ == '__main__':
    main()

